
#include "standard.h"
#include "setjmp.h"

/*
 *  input        parses as   resulting evalstack
 *
 *
 *  "KP41BB" locator me coord distance 1000 / 10 > assert "PP00AA" locator ...
 *
 *  "KP41BB"     string      string "KP41BB"
 *  locator      function    struct coord
 *  me           magic       struct coord, magic 'me'
 *  coord        function    struct coord, struct coord
 *  distance     function    integer 5700          (whatever, meters)
 *  1000         integer     integer 5700, integer 1000
 *  /            function    integer 5             (5700 / 1000)
 *  10           integer     integer 5, integer 10
 *  >            function    boolean TRUE          (10 > 5)
 *  assert       function    empty
 *  "PP00AA"     string      string "PP00AA"
 *  locator      function  ...
 *                       ... boolean FALSE
 *  assert       function    empty                 (and FALSE status returned)
 *  foo                                            (not reached)
 *
 *
 *  another:  1 return foo bar
 *
 *  1            integer     integer 1
 *  return       function    empty                 (TRUE status returned)
 *  foo                                            (not reached)
 *  bar                                            (not reached)
 *
 *
 *  one more:  inputs 0b0010010 ^ 0b0010011 & 0b0010011 ==
 *
 *  inputs       function     integer 0b10101010010  (digital inputs collected)
 *  0b0010010    integer      integers 0b10101010010, 0b0010010
 *  ^            function     integer 0b10101000000
 *  0b0010011    integer      integers 0b10101000000, 0b0010011
 *  &            function     integer 0
 *  0b0010011    integer      integers 0, 0b0010011
 *  ==           function     boolean 0             (FALSE returned at end)
 *
 *
 *  31 store 30 load
 *
 *  load/store permanent (until restart/reboot) state.
 */

static jmp_buf bailout;

static void
oops()
{
	longjmp(bailout, 1);
}

typedef
struct item {
	enum {
		NONE,
		MAGIC,
		BOOLEAN, INTEGER, STRING,
		COORD,
	} type;
	union {
		word magic;
		word w;
		long l;
		dword dw;
		byte *str;             /* or [32] ? */
		struct coord coord;
	} u;
}
	item;

static item evalstack[16], *Etop;

static void
op_integer(byte *token)
{
	Etop->u.i = atol(token);
}

static void
op_string(byte *token)
{
	Etop->u.str = token;  /* XXX requires care */
}

static void
op_add()
{
	Etop[1].u.i += Etop->u.i;
}

static void
op_coord()
{
	switch (Etop->type) {

	case MAGIC:                   /* 'me', 'launch' etc */
		switch (Etop->u.magic) {

		case MAGIC_ME:
			Etop->u.coord = gps;
			break;
		}
		break;

	case STRING:             /* coordinate string, station call ? */
		break;
	}
}

		byte *token;
		void *handler;
		int stk_change;
		int type;
		int extra;

	{ "+",     op_add,     +1, INTEGER, },
	{ "coord", op_coord,    0, COORD, },
	{ NULL,    op_integer, -1, INTEGER, },
	{ NULL,    op_string,  -1, STRING, },
	{ "me",    op_magic,   -1, MAGIC, MAGIC_ME },

bool
interpret_predicate(byte *prog)
{
	if (setjmp(bailout))
		return (FALSE);

	Etop = __ENDOF(evalstack) - 1;
	Etop->type = NONE;

	...

		p = lookup(token);
		if (!p)
			oops();

		if (p->stk_change < 0 && --Etop < evalstack)
			oops();

		if (p->type == MAGIC)
			Etop->u.magic = p->extra;
		else
			fncall(token, p->handler);

		Etop->type = p->type;

		if (p->stk_change > 0 && ++Etop >= __ENDOF(evalstack))
			oops();

		...
}

