
#include "standard.h"

#define KISS_CRC   0xA0
#define KISS_SMACK 0x80
#define KISS_RMNC  0x20


#define crc_SMACK(p,n,crc) crc_generic(p,crc,0xA001,n)
#define crc_RMNC(p,n,crc)  crc_generic_RMNC(n,crc,p,0x8408)

#define ld_word_moto(a) (*(word *) (a))

/* port/command byte must still remain in p[0] */

int
verify_kiss_crc(byte *p, word n)
{
	word calc, rcvd;

	switch (*p & KISS_CRC) {

	/* what about when both are set ? :) */
	default:
		return (0);

	case KISS_SMACK:
		if (n < 1 + 2)
			return (-1);

		rcvd = ld_word_intel(&p[n - 2]);
		calc = crc_SMACK(p, n - 2, 0);
		break;

	case KISS_RMNC:
		if (n < 1 + 2)
			return (-1);

		rcvd = ld_word_moto(&p[n - 2]);
		calc = crc_RMNC(p, n - 2, ~0);
		break;
	}
	if (calc != rcvd)
		return (-1);

	return (*p & KISS_CRC);
}

/* returned word must be sent LSByte first, it is pre-swapped here */

word
calc_kiss_crc(byte cmd, byte *p, word len)
{
	word crc;

	if (len) {
		switch (cmd & KISS_CRC) {
		case KISS_SMACK:
			crc = crc_SMACK(p, len, crc_SMACK(&cmd, 1, 0));

			return (crc);         /* not swapped, LSByte to go 1st */

		case KISS_RMNC:
			crc = crc_RMNC(p, len, crc_RMNC(&cmd, 1, ~0));

			return ((crc << 8) + (crc >> 8));
		}
	}
	doh("kisrc");
}

#if 0 /* in crc.s --- argument order is a femto-optimisation */

static word
crc_generic_RMNC(word n, word crc, byte *p, word poly)
{

	word x, ox;

	/* XXX turn to assembly someday.
	 * XXX ugh the extra FFFF and martyfeldman shifting */

	do {
		x = crc >> 8;
		x ^= ~0;
		x ^= *p++;

#define BIT ox = x; x >>= 1; if (ox & 1) x ^= poly;

		BIT; BIT; BIT; BIT
		BIT; BIT; BIT; BIT

		crc <<= 8;
		crc ^= x;

	} while (--n);

	return (crc);
}

#endif

