
;
;  MC145040 or 050 or family hanging from QSPI.
;  Connections to D9, except PCS0 directly to cpu leg.
;  if needed, 4 output bits (aux) can be latched at PCS0 rising edge
;  into lower half of 74595 for example.
;

	include regs.inc

	section .text
	align 1

	global _init_spi_adc, _spi_adc_value
	global _write_spi_aux

_write_spi_aux       ; void write_spi_aux(byte fourbits);
	pshm k
	ldab # FF
	tbxk
	ldx  # TRAM
	ted
	andb # 0Fh       ; 4 bits available
L3
	ldaa 1,x         ; LSByte of transmit word
	anda # ~0Fh      ; just the mux bits please
	aba              ; add new aux bits
	staa 1,x         ; it might stutter until all words are updated
	aix  #+2
	cpx  # TRAM + 24 ; into all 12 transfers XXX what a waste
	blo  L3
	pulm k
	rts


_spi_adc_value       ; word spi_adc_value(word channel);
	pshm k
	ldab # FF
	tbxk
	asle             ; index to word offset
	ldx  # RRAM
		; ldd 22,x       ; AN11
		; cpd # 007Fh
		; blo error
		; cpd # 0081h
		; bhi error
	ldd  e,x         ; unused msbits are automatically zero
	pulm k
	rts


_init_spi_adc        ; void init_spi_adc(enum { MC145040, MC145050 } );
	pshm k
	ldab # FF
	tbek
	tbxk

	; control and transmit ram

	ldx  # CRAM
	ldaa # 30h       ; CONT=0 BITSE=0=8 DT=1 DSCK=1 PCSx=0
	ldab # 10h       ; 4-bit mux select for 8-bit AN1, 0001'0000
	tste
	beq  L0          ; MC145040 or 041
	ldaa # 70h       ; MC145050 or 051, so BITSE=1=BITS
	ldab # 40h       ; 4-bit mux select for 10-bit AN1, 00'0100'0000
L0
	staa 0,x
	aix  #+1
	cpx  # CRAM + 12 ; for all 12 a/d channels used
	blo  L0

	ldx  # TRAM
	clra             ; D has bits for AN1
	clre             ; mux select
L1
	ade              ; mux select is always one ahead
	ste  0,x
	aix  #+2
	cpx  # TRAM + 24 ; for all 12 a/d channels used
	blo  L1
	clrw -2,x        ; fix last mux select, read AN11 and start AN0

	; pins
	; note WOMS is cleared, no pullups required

	ldab # 8Eh       ; PCS3,2,1 and MISO are inputs
	stab DDRQS
	ldab # 88h       ; (TXD,)PCS0 idle high; SCK,MOSI idle low
	stab PORTQS

	ldab # 0Fh       ; PCS0,SCK,MOSI,MISO owned by QSPI
	stab QSPAR

	; SPBR  FFh   33    kHz              baud = 16.777 MHz / 2 / SPBR
	;        8h    1.05 MHz
	;        9h    0.93 MHz    ok
	; DSCKL 7Fh    7.6  usec             usec = DSCKL / 16.777 MHz
	;       43h    4.0  usec   ok
	; DTL   FFh  486    usec             usec = 32 * DTL / 16.777 MHz
	;       15h   40    usec   ok

	ldd  # 0A809h    ; MSTR=1 WOMQ=0 BITS=10 CPOL=0 CPHA=0 SPBR=9=1MHz
	std  SPCR0       ; BITS effective only if BITSE=1, above.

	ldd  # 4B00h     ; WREN=1 ENDQP=11 NEWQP=0
	std  SPCR2       ; SPIFIE WREN WRTO ENDQP undefined NEWQP

	clrw SPCR3       ; HMIE LOOPQ etc all 0. also clears SPSR

	ldd  # 0C315h    ; SPE=1 DSCKL=43h DTL=15h
	std  SPCR1       ; GO !

	pulm k
	rts              ; and QSPI keeps looping in background

